# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['creole',
 'creole.emitter',
 'creole.html_tools',
 'creole.parser',
 'creole.rest_tools',
 'creole.shared',
 'creole.tests',
 'creole.tests.utils']

package_data = \
{'': ['*']}

install_requires = \
['docutils>=0.16,<0.17']

entry_points = \
{'console_scripts': ['creole2html = creole.cmdline:cli_creole2html',
                     'html2creole = creole.cmdline:cli_html2creole',
                     'html2rest = creole.cmdline:cli_html2rest',
                     'html2textile = creole.cmdline:cli_html2textile']}

setup_kwargs = {
    'name': 'python-creole',
    'version': '1.4.0',
    'description': 'python-creole is an open-source (GPL) markup converter in pure Python for: creole2html, html2creole, html2ReSt, html2textile',
    'long_description': '= about =\n\npython-creole is a OpenSource (GPL) Python lib for converting markups.\npython-creole is pure python. No external libs needed.\n\nCompatible Python Versions (see also [[https://github.com/jedie/python-creole/blob/master/.travis.yml|.travis.yml]]):\n * 2.7.x, 3.3, 3.4, 3.5, 3.6\n * PyPy (Python v2 and v3)\n\n\nExisting converters:\n\n* creole -> html\n* html -> creole markup\n* reSt -> html (for clean html code)\n* html -> reStructuredText markup (only a subset of reSt supported)\n* html -> textile markup (not completed yet)\n\nThe creole2html part based on the creole markup parser and emitter from the MoinMoin project by Radomir Dopieralski and Thomas Waldmann.\n\n| {{https://travis-ci.org/jedie/python-creole.svg|Build Status on travis-ci.org}} | [[https://travis-ci.org/jedie/python-creole/|travis-ci.org/jedie/python-creole]] |\n| {{https://coveralls.io/repos/jedie/python-creole/badge.svg|Coverage Status on coveralls.io}} | [[https://coveralls.io/r/jedie/python-creole|coveralls.io/r/jedie/python-creole]] |\n| {{https://landscape.io/github/jedie/python-creole/master/landscape.svg|Status on landscape.io}} | [[https://landscape.io/github/jedie/python-creole/master|landscape.io/github/jedie/python-creole/master]] |\n\n\n= install =\n\nPython packages available on: [[http://pypi.python.org/pypi/python-creole/]]\n\n{{{\n~$ pip install python-creole\n}}}\n\nTo setup a virtualenv via Poetry, see {{{unittests}}} section below.\n\n== dependencies ==\n\nFor the most parts ({{{creole2html}}} and {{{html2creole}}}) no external libraries are needed.\n\nFor all functionality (and running the unittests) these modules are needed:\n\n* [[http://pypi.python.org/pypi/docutils/|docutils]] (for the ReStructuredText stuff)\n* [[http://pypi.python.org/pypi/textile/|textile]] (for html2textile tests)\n\n\n= example =\n\n== creole2html ==\nConvert creole markup to html code:\n{{{\n>>> from creole import creole2html\n>>> creole2html("This is **creole //markup//**")\nu\'<p>This is <strong>creole <i>markup</i></strong></p>\\n\'\n}}}\n\n\n== html2creole ==\nConvert html code back into creole markup:\n{{{\n>>> from creole import html2creole\n>>> html2creole(u\'<p>This is <strong>creole <i>markup</i></strong></p>\\n\')\nu\'This is **creole //markup//**\'\n}}}\n\n\n== rest2html ==\nConvert ReStructuredText into clean html code (needs [[http://pypi.python.org/pypi/docutils/|docutils]]):\n{{{\n>>> from creole.rest2html.clean_writer import rest2html\n>>> rest2html(u"A ReSt link to `PyLucid CMS <http://www.pylucid.org>`_ :)")\nu\'<p>A ReSt link to <a href="http://www.pylucid.org">PyLucid CMS</a> :)</p>\\\\n\'\n}}}\n(more information: [[https://github.com/jedie/python-creole/wiki/rest2html|rest2html wiki page]])\n\n\n== html2rest ==\nConvert html code into ReStructuredText markup:\n{{{\n>>> from creole import html2rest\n>>> html2rest(u\'<p>This is <strong>ReStructuredText</strong> <em>markup</em>!</p>\')\nu\'This is **ReStructuredText** *markup*!\'\n}}}\n\n\n== html2textile ==\nConvert html code into textile markup\n{{{\n>>> from creole import html2textile\n>>> html2textile(u\'<p>This is <strong>textile <i>markup</i></strong>!</p>\')\nu\'This is *textile __markup__*!\'\n}}}\n\nSee also: [[http://github.com/jedie/python-creole/blob/master/demo.py]]\n\n= Image size additional =\n\nYou can pass image width/height in image tags, e.g.:\n{{{\n>>> from creole import creole2html\n>>> creole_markup="""{{foobar.jpg|image title|90x160}}"""\n>>> creole2html(creole_markup)\n\'<p><img src="foobar.jpg" title="image title" alt="image title" width="90" height="160" /></p>\'\n}}}\n\nThe third part ({{{90x160}}}) is not in creole standard, you can force a //strict// mode, e.g.:\n{{{\n>>> creole2html(creole_markup, strict=True)\n\'<p><img src="foobar.jpg" title="image title|90x160" alt="image title|90x160" /></p>\'\n}}}\n\n= Source code highlighting support =\n\nYou can find a example macro which highlight source code thanks to the pygments\nlibrary. It is located here: [[https://github.com/jedie/python-creole/blob/master/creole/shared/example_macros.py|/creole/shared/example_macros.py]].\nHere is how to use it:\n\n{{{\n>>> from creole import creole2html\n>>> from creole.shared.example_macros import code\n>>> creole_markup="""<<code ext=".py">>#some code\\nprint(\'coucou\')\\n<</code>>"""\n>>> creole2html(creole_markup, macros={\'code\': code})\n}}}\n\n= commandline interface =\n\nIf you have python-creole installed, you will get these simple CLI scripts:\n* creole2html\n* html2creole\n* html2rest\n* html2textile\n\nHere the {{{--help}}} output from {{{html2creole}}}:\n{{{\n$ html2creole --help\nusage: html2creole [-h] [-v] [--encoding ENCODING] sourcefile destination\n\npython-creole is an open-source (GPL) markup converter in pure Python for:\ncreole2html, html2creole, html2ReSt, html2textile\n\npositional arguments:\n  sourcefile           source file to convert\n  destination          Output filename\n\noptional arguments:\n  -h, --help           show this help message and exit\n  -v, --version        show program\'s version number and exit\n  --encoding ENCODING  Codec for read/write file (default encoding: utf-8)\n}}}\n\nExample to convert a html file into a creole file:\n{{{\n$ html2creole foobar.html foobar.creole\n}}}\n\n= documentation =\n\nWe store documentation/examples into the project wiki:\n\n* [[https://github.com/jedie/python-creole/wiki]]\n\nHow to handle unknown html tags in html2creole:\n\n* [[https://github.com/jedie/python-creole/wiki/Unknown-Html-Tags]]\n\nContributers should take a look at this page:\n\n* [[https://github.com/jedie/python-creole/wiki/Developer-Info]]\n\nCreole Markup Cheat Sheet can be found here: http://www.wikicreole.org/wiki/CheatSheet\n\n{{http://www.wikicreole.org/imageServlet?page=CheatSheet%2Fcreole_cheat_sheet.png&width=340|Creole Markup Cheat Sheet}}\n\n== unittests ==\n\nSetup a dev. environment with [[https://python-poetry.org/|Poetry]] and run Tests via PyTest/Tox:\n\n{{{\n# clone repository (or use your fork):\n~$ git clone https://github.com/jedie/python-creole.git\n~$ cd python-creole\n\n# install or update poetry:\n~/python-crole$ make install-poetry\n\n# install python-creole via poetry:\n~/python-crole$ make install\n\n# Run pytest:\n~/python-crole$ make pytest\n\n# Run pytest via tox with all environments:\n~/python-crole$ make tox\n\n# Run pytest via tox with one Python version:\n~/python-crole$ make tox-py38\n~/python-crole$ make tox-py37\n~/python-crole$ make tox-py36\n}}}\n\n\n== Use creole in README ==\n\nWith python-creole you can convert a README on-the-fly from creole into ReStructuredText in setup.py\nHow to do this, read: https://github.com/jedie/python-creole/wiki/Use-In-Setup\n\nNote: In this case you must install **docutils**! See above.\n\n\n= History =\n\n* *dev* - [[https://github.com/jedie/python-creole/compare/v1.4.0...master|compare v1.4.0...master]]\n** TBC\n* v1.4.0 - 2020-01-19 - [[https://github.com/jedie/python-creole/compare/v1.3.2...master|compare v1.3.2...master]]\n** modernize project:\n*** use poetry\n*** Add a {{{Makefile}}}\n*** use pytest and tox\n*** remove Python v2 support\n*** Test with Python v3.6, v3.7 and v3.8\n* v1.3.2 - 2018-02-27 - [[https://github.com/jedie/python-creole/compare/v1.3.1...v1.3.2|compare v1.3.1...v1.3.2]]\n** Adding optional img size to creole2html and html2creole contributed by [[https://github.com/JohnAD|John Dupuy]]\n** run tests also with python 3.5 and 3.6\n* v1.3.1 - 2015-08-15 - [[https://github.com/jedie/python-creole/compare/v1.3.0...v1.3.1|compare v1.3.0...v1.3.1]]\n** Bugfix for "Failed building wheel for python-creole"\n* v1.3.0 - 2015-06-02 - [[https://github.com/jedie/python-creole/compare/v1.2.2...v1.3.0|compare v1.2.2...v1.3.0]]\n** Refactory internal file structure\n** run unittests and doctests with nose\n** Refactor CLI tests\n** skip official support for Python 2.6\n** small code cleanups and fixes.\n** use **json.dumps()** instead of **repr()** in some cases\n* v1.2.2 - 2015-04-05 - [[https://github.com/jedie/python-creole/compare/v1.2.1...v1.2.2|compare v1.2.1...v1.2.2]]\n** Bugfix textile unittests if url scheme is unknown\n** migrate google-code Wiki to github and remove google-code links\n* v1.2.1 - 2014-09-14 - [[https://github.com/jedie/python-creole/compare/v1.2.0...v1.2.1|compare v1.2.0...v1.2.1]]\n** Use origin PyPi code to check generated reStructuredText in setup.py\n** Update unitest for textile v2.1.8\n* v1.2.0 - 2014-05-15 - [[https://github.com/jedie/python-creole/compare/v1.1.1...v1.2.0|compare v1.1.1...v1.2.0]]\n** NEW: Add {{{<<code>>}}} example macro (Source code highlighting with pygments) - implemented by Julien Enselme\n** NEW: Add {{{<<toc>>}}} macro to create a table of contents list\n** Bugfix for: AttributeError: \'CreoleParser\' object has no attribute \'_escaped_char_repl\'\n** Bugfix for: AttributeError: \'CreoleParser\' object has no attribute \'_escaped_url_repl\'\n** API Change: Callable macros will raise a TypeError instead of create a DeprecationWarning (Was removed in v0.5) \n* v1.1.1 - 2013-11-08\n** Bugfix: Setup script exited with error: can\'t copy \'README.creole\': doesn\'t exist or not a regular file\n* v1.1.0 - 2013-10-28\n** NEW: Simple commandline interface added.\n* v1.0.7 - 2013-08-07\n** Bugfix in \'clean reStructuredText html writer\' if docutils => v0.11 used.\n** Bugfix for PyPy 2.1 usage\n* v1.0.6 - 2012-10-15\n** Security fix in rest2html: Disable "file_insertion_enabled" and "raw_enabled" as default.\n* v1.0.5 - 2012-09-03\n** made automatic protocol links more strict: Only whitespace before and at the end are allowed.\n** Bugfix: Don\'t allow {{{ftp:/broken}}} (Only one slash) to be a link.\n* v1.0.4 - 2012-06-11\n** html2rest: Handle double link/image substitution and raise better error messages\n** Bugfix in unittests (include test README file in python package).  Thanks to Wen Heping for reporting this.\n* v1.0.3 - 2012-06-11\n** Bugfix: {{{AttributeError: \'module\' object has no attribute \'interesting_cdata\'}}} from HTMLParser patch. Thanks to Wen Heping for reporting this.\n** Fix a bug in get_long_description() ReSt test for Py3k and his unittests.\n** Use Travis CI, too.\n* v1.0.2 - 2012-04-04\n** Fix "[[https://github.com/jedie/python-creole/issues/6|AttributeError: \'NoneType\' object has no attribute \'parent\']]" in html2creole.\n* v1.0.1 - 2011-11-16\n** Fix "[[https://github.com/jedie/python-creole/issues/5|TypeError: expected string or buffer]]" in rest2html.\n** [[https://github.com/jedie/python-creole/commit/e8422f944709a5f8c2c6a8c8a58a84a92620f035|Bugfix in exception handling.]]\n* v1.0.0 - 2011-10-20\n** Change API: Replace \'parser_kwargs\' and \'emitter_kwargs\' with separate arguments. (More information on [[https://github.com/jedie/python-creole/wiki/API|API Wiki Page]])\n* v0.9.2\n** Turn zip_safe in setup.py on and change unittests API.\n* v0.9.1\n** Many Bugfixes, tested with CPython 2.6, 2.7, 3.2 and PyPy v1.6\n* v0.9.0\n** Add Python v3 support (like http://python3porting.com/noconv.html strategy)\n** move unittests into creole/tests/\n** Tested with Python 2.7.1, 3.2 and PyPy v1.6.1 15798ab8cf48 jit\n* v0.8.5\n** Bugfix in html2creole: ignore links without href\n* v0.8.4\n** Bugfix in html parser if list tag has attributes: https://code.google.com/p/python-creole/issues/detail?id=19#c4\n* v0.8.3\n** Better error message if given string is not unicode: https://code.google.com/p/python-creole/issues/detail?id=19\n* v0.8.2\n** Bugfix in get_long_description() error handling (//local variable \'long_description_origin\' referenced before assignment//)\n* v0.8.1\n** Bugfix for installation under python 2.5\n** Note: [[https://github.com/jedie/python-creole/wiki/Use-In-Setup|setup helper]] changed: rename {{{GetLongDescription(...)}}} to {{{get_long_description(...)}}}\n* v0.8\n** New GetLongDescription() helper for setup.py, see: https://github.com/jedie/python-creole/wiki/Use-In-Setup\n* v0.7.3\n** Bugfix in html2rest:\n*** table without {{{<th>}}} header\n*** new line after table\n*** create reference hyperlinks in table cells intead of embedded urls.\n*** Don\'t always use raise_unknown_node()\n** Add child content to raise_unknown_node()\n* v0.7.2\n** Activate {{{----}}} to {{{<hr>}}} in html2rest\n** Update demo.py\n* v0.7.1\n** Bugfix if docutils are not installed\n** API change: rest2html is now here: {{{ from creole.rest2html.clean_writer import rest2html }}}\n* v0.7.0\n** **NEW**: Add a html2reStructuredText converter (only a subset of reSt supported)\n* v0.6.1\n** Bugfix: separate lines with one space in "wiki style line breaks" mode\n* v0.6\n** **NEW**: html2textile converter\n** some **API changed**!\n* v0.5\n** **API changed**:\n*** Html2CreoleEmitter optional argument \'unknown_emit\' takes now a callable for handle unknown html tags.\n*** No macros used as default in creole2html converting.\n*** We remove the support for callable macros. Only dict and modules are allowed.\n** remove unknown html tags is default behaviour in html2creole converting.\n** restructure and cleanup sourcecode files.\n* v0.4\n** only emit children of empty tags like div and span (contributed by Eric O\'Connell)\n** remove inter wiki links and doesn\'t check the protocol\n* v0.3.3\n** Use <tt> when ~{~{~{ ... ~}~}~} is inline and not <pre>, see: [[http://forum.pylucid.org/viewtopic.php?f=3&t=320|PyLucid Forum Thread]]\n** Bugfix in html2creole: insert newline before new list. TODO: apply to all block tags: [[http://code.google.com/p/python-creole/issues/detail?id=16#c5|issues 16]]\n* v0.3.2\n** Bugfix for spaces after Headline: [[https://code.google.com/p/python-creole/issues/detail?id=15|issues 15]]\n* v0.3.1\n** Make argument \'block_rules\' in Parser() optional\n* v0.3.0\n** creole2html() has the optional parameter \'blog_line_breaks\' to switch from default blog to wiki line breaks\n* v0.2.8\n** bugfix in setup.py\n* v0.2.7\n** handle obsolete non-closed <br> tag\n* v0.2.6\n** bugfix in setup.py\n** Cleanup DocStrings\n** add unittests\n* v0.2.5\n** creole2html: Bugfix if "--", "~/~/" etc. stands alone, see also: [[http://code.google.com/p/python-creole/issues/detail?id=12|issues 12]]\n** Note: bold, italic etc. can\'t cross line any more.\n* v0.2.4\n** creole2html: ignore file extensions in image tag\n*** see also: [[http://code.google.com/p/python-creole/issues/detail?id=7|issues 7]]\n* v0.2.3\n** html2creole bugfix/enhanced: convert image tag without alt attribute:\n*** see also: [[http://code.google.com/p/python-creole/issues/detail?id=6|issues 6]]\n*** Thanks Betz Stefan alias \'encbladexp\'\n* v0.2.2\n** html2creole bugfix: convert {{{<a href="/url/">Search & Destroy</a>}}}\n* v0.2.1\n** html2creole bugfixes in:\n*** converting tables: ignore tbody tag and better handling p and a tags in td\n*** converting named entity\n* v0.2\n** remove all django template tag stuff: [[http://code.google.com/p/python-creole/issues/detail?id=3|issues 3]]\n** html code always escaped\n* v0.1.1\n** improve macros stuff, patch by Vitja Makarov: [[http://code.google.com/p/python-creole/issues/detail?id=2|issues 2]]\n* v0.1.0\n** first version cut out from [[http://www.pylucid.org|PyLucid CMS]]\n\nfirst source code was written 27.11.2008: [[http://www.python-forum.de/viewtopic.php?f=3&t=16742|Forum thread (de)]]\n\n\n== Project links ==\n\n| GitHub        | [[https://github.com/jedie/python-creole]]\n| Wiki          | [[https://github.com/jedie/python-creole/wiki]]\n| PyPi          | [[https://pypi.org/project/python-creole/]]\n\n\n== donation\n\n* [[https://www.paypal.me/JensDiemer|paypal.me/JensDiemer]]\n* [[https://flattr.com/submit/auto?uid=jedie&url=https%3A%2F%2Fgithub.com%2Fjedie%2Fpython-creole%2F|Flattr This!]]\n* Send [[http://www.bitcoin.org/|Bitcoins]] to [[https://blockexplorer.com/address/1823RZ5Md1Q2X5aSXRC5LRPcYdveCiVX6F|1823RZ5Md1Q2X5aSXRC5LRPcYdveCiVX6F]]\n',
    'author': 'Jens Diemer',
    'author_email': 'python-creole@jensdiemer.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jedie/python-creole/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
